@@ -31,6 +31,7 @@
 #include "keycache.h"
 #include "mymapitags.h"
 #include "recipient.h"
+#include "recipientmanager.h"
 #include "windowmessages.h"
 
 #include <gpgme++/context.h>
@@ -91,13 +92,7 @@ int
 CryptController::collect_data ()
 {
   TSTART;
-  /* Get the attachment info and the body.  We need to do this before
-     creating the engine's filter because sending the cancel to
-     the engine with nothing for the engine to process.  Will result
-     in an error. This is actually a bug in our engine code but
-     we better avoid triggering this bug because the engine
-     sometimes hangs.  Fixme: Needs a proper fix. */
-
+  int count = m_mail->plainAttachments ().size ();
 
   /* Take the Body from the mail if possible. This is a fix for
      GnuPG-Bug-ID: T3614 because the body is not always properly
@@ -108,19 +103,7 @@ CryptController::collect_data ()
       xfree (body);
       body = nullptr;
     }
-
-  LPMESSAGE message = m_mail->isCryptoMail() ?
-                      get_oom_base_message (m_mail->item ()) :
-                      get_oom_message (m_mail->item ());
-  if (!message)
-    {
-      log_error ("%s:%s: Failed to get message.",
-                 SRCNAME, __func__);
-    }
-
-  auto att_table = mapi_create_attach_table (message, 0);
-  int n_att_usable = count_usable_attachments (att_table);
-  if (!n_att_usable && !body)
+  if (!count && !body)
     {
       if (!m_mail->isDraftEncrypt())
         {
@@ -128,15 +111,13 @@ CryptController::collect_data ()
                              utf8_gettext ("Can't encrypt / sign an empty message."),
                              utf8_gettext ("GpgOL"), MB_OK);
         }
-      gpgol_release (message);
-      mapi_release_attach_table (att_table);
       xfree (body);
       TRETURN -1;
     }
 
   bool do_inline = m_mail->getDoPGPInline ();
 
-  if (n_att_usable && do_inline)
+  if (count && do_inline)
     {
       log_debug ("%s:%s: PGP Inline not supported for attachments."
                  " Using PGP MIME",
@@ -164,24 +145,16 @@ CryptController::collect_data ()
   sink->writefnc = sink_data_write;
 
   /* Collect the mime strucutre */
-  int err = add_body_and_attachments (sink, message, att_table, m_mail,
-                                      body, n_att_usable);
+  int err = add_body_and_attachments (sink, m_mail, body);
   xfree (body);
 
   if (err)
     {
       log_error ("%s:%s: Collecting body and attachments failed.",
                  SRCNAME, __func__);
-      gpgol_release (message);
-      mapi_release_attach_table (att_table);
       TRETURN -1;
     }
 
-  /* Message is no longer needed */
-  gpgol_release (message);
-
-  mapi_release_attach_table (att_table);
-
   /* Set the input buffer to start. */
   m_input.seek (0, SEEK_SET);
   TRETURN 0;
@@ -921,7 +894,40 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
   /* Start a WKS check if necessary. */
   WKSHelper::instance()->start_check (m_mail->getSender ());
 
-  int ret = resolve_keys ();
+  int ret = 0;
+
+  if (m_mail->copyParent ())
+    {
+      /* Bypass resolving if we are working on a split mail */
+      m_recipients = m_mail->getCachedRecipients ();
+      m_signer_keys = m_mail->getSigningKeys ();
+
+      if (m_recipients.size () && m_recipients[0].keys ().size ())
+        {
+          m_proto = m_recipients[0].keys ()[0].protocol ();
+        }
+      else if (m_signer_keys.size ())
+        {
+          m_proto = m_signer_keys[0].protocol ();
+        }
+
+      m_enc_keys.clear ();
+      for (const auto &recp: m_recipients)
+        {
+          const auto &keys = recp.keys ();
+          m_enc_keys.insert (m_enc_keys.end (), keys.begin (), keys.end ());
+        }
+
+      if ((opt.enable_debug & DBG_DATA))
+        {
+          log_data ("Encrypting to: ");
+          Recipient::dump (m_recipients);
+        }
+    }
+  else
+    {
+      ret = resolve_keys ();
+    }
 
   if (ret == -1)
     {
@@ -936,33 +942,20 @@ CryptController::do_crypto (GpgME::Error &err, std::string &r_diag)
       TRETURN -2;
     }
 
-  /* If we need to send multiple emails we jump back from
-     here into the main event loop. Copy the mail object
-     and send it out mutiple times. */
-  if (opt.splitBCCMails)
+  if (!m_mail->copyParent ())
     {
-      bool foundOneNormalRecp = false;
-      bool foundOneBCCRecp = false;
-      for (const auto &recp: m_recipients)
+      RecipientManager mngr (m_recipients, m_signer_keys);
+      if (mngr.getRequiredMails () > 1)
         {
-          if (foundOneBCCRecp && foundOneNormalRecp)
-            {
-              log_debug ("%s:%s: Have both BCC and normal recipients."
-                         " Need to send multiple mails.",
-                         SRCNAME, __func__);
-              do_in_ui_thread_async (SEND_MULTIPLE_MAILS, m_mail);
-              /* Cancel the crypto of this mail */
-              TRETURN -3;
-            }
-          if (recp.type() == Recipient::olCC ||
-              recp.type() == Recipient::olTo)
-            {
-              foundOneNormalRecp = true;
-            }
-          if (recp.type() == Recipient::olBCC)
-            {
-              foundOneBCCRecp = true;
-            }
+          log_dbg ("More then one mail required for this recipient selection.");
+          /* If we need to send multiple emails we jump back from
+             here into the main event loop. Copy the mail object
+             and send it out mutiple times. */
+          do_in_ui_thread_async (SEND_MULTIPLE_MAILS, m_mail);
+          /* Cancel the crypto of this mail this continues
+             in Mail::splitAndSend_o */
+          wm_unregister_pending_op (m_mail);
+          TRETURN -3;
         }
     }
 
