@@ -2501,6 +2501,34 @@ get_unique_id (LPDISPATCH mail, int create, const char *uuid)
   TRETURN newuid;
 }
 
+char *
+reset_unique_id (LPDISPATCH mail)
+{
+  TSTART;
+  char *newuid = generate_uid ();
+
+  int ret = put_pa_string (mail, GPGOL_UID_DASL, newuid);
+  if (ret)
+    {
+      log_debug ("%s:%s: failed to set uid '%s' for '%p'",
+                 SRCNAME, __func__, newuid, mail);
+      xfree (newuid);
+      TRETURN NULL;
+    }
+  TRETURN newuid;
+}
+
+std::string
+get_unique_id_s (LPDISPATCH mail, int create, const char *uuid)
+{
+  char *val = get_unique_id (mail, create, uuid);
+  if (val)
+    {
+      return val;
+    }
+  return std::string (val);
+}
+
 HWND
 get_active_hwnd ()
 {
@@ -3427,13 +3455,20 @@ set_oom_recipients (LPDISPATCH item, const std::vector<Recipient> &recps)
       TRETURN -1;
     }
 
-  /* First clear out the current recipients. */
-  int ret = invoke_oom_method (oom_recps.get (), "RemoveAll", nullptr);
+  int count = get_oom_int (oom_recps.get (), "Count");
 
-  if (ret)
+  for (int i = 1; i <= count; i++)
     {
-      STRANGEPOINT;
-      TRETURN ret;
+      /* First clear out the current recipients. */
+      int ret = invoke_oom_method_with_int (oom_recps.get (),
+                                            "Remove", 1,
+                                            nullptr);
+
+      if (ret)
+        {
+          STRANGEPOINT;
+          TRETURN ret;
+        }
     }
 
   for (const auto &recp: recps)
@@ -3446,9 +3481,9 @@ set_oom_recipients (LPDISPATCH item, const std::vector<Recipient> &recps)
         }
       VARIANT result;
       VariantInit (&result);
-      ret = invoke_oom_method_with_string (oom_recps.get (), "Add",
-                                           recp.mbox ().c_str (),
-                                           &result);
+      int ret = invoke_oom_method_with_string (oom_recps.get (), "Add",
+                                               recp.mbox ().c_str (),
+                                               &result);
       if (ret)
         {
           log_err ("Failed to add recipient.");
@@ -3656,7 +3691,89 @@ show_folder_select ()
   TRETURN nullptr;
 }
 
-LPDISPATCH oApp ()
+LPDISPATCH
+oApp ()
 {
   return GpgolAddin::get_instance()->get_application();
 }
+
+BSTR utf8_to_bstr (const char *string)
+{
+  wchar_t *tmp = utf8_to_wchar (string);
+  BSTR bstring = tmp ? SysAllocString (tmp) : NULL;
+  xfree (tmp);
+  if (!bstring)
+    {
+      log_error_w32 (-1, "%s:%s: SysAllocString failed", SRCNAME, __func__);
+      TRETURN nullptr;
+    }
+  TRETURN bstring;
+}
+
+int
+oom_save_as (LPDISPATCH obj, const char *path, oomSaveAsType type)
+{
+  if (!obj || !path)
+    {
+      /* invalid arguments */
+      STRANGEPOINT;
+      TRETURN -1;
+    }
+  /* Params are first path and then type as optional. With
+     COM Marshalling this means that param 1 is the path
+     and 0 is the type. */
+  VARIANT aVariant[2];
+  VariantInit(aVariant);
+  VariantInit(aVariant + 1);
+
+  DISPPARAMS dispparams;
+  dispparams.rgvarg = aVariant;
+  dispparams.rgvarg[0].vt = VT_INT;
+  dispparams.rgvarg[0].intVal = (int) type;
+  dispparams.rgvarg[1].vt = VT_BSTR;
+  dispparams.rgvarg[1].bstrVal = utf8_to_bstr (path);
+  dispparams.cArgs = 2;
+  dispparams.cNamedArgs = 0;
+
+  int rc = invoke_oom_method_with_parms (obj, "SaveAs", nullptr, &dispparams);
+
+  if (rc)
+    {
+      log_err ("Failed to call SaveAs");
+    }
+  VariantClear(aVariant);
+  VariantClear(aVariant + 1);
+
+  return rc;
+}
+
+int
+oom_save_as_file (LPDISPATCH obj, const char *path)
+{
+  if (!obj || !path)
+    {
+      /* invalid arguments */
+      STRANGEPOINT;
+      TRETURN -1;
+    }
+  VARIANT aVariant[1];
+  VariantInit(aVariant);
+
+  DISPPARAMS dispparams;
+  dispparams.rgvarg = aVariant;
+  dispparams.rgvarg[0].vt = VT_BSTR;
+  dispparams.rgvarg[0].bstrVal = utf8_to_bstr (path);
+  dispparams.cArgs = 1;
+  dispparams.cNamedArgs = 0;
+
+  int rc = invoke_oom_method_with_parms (obj, "SaveAsFile",
+                                         nullptr, &dispparams);
+
+  if (rc)
+    {
+      log_err ("Failed to call SaveAsFile");
+    }
+  VariantClear(aVariant);
+
+  return rc;
+}
